/***

The MIT License

Copyright (c) 2018 Teppei Tamra (TAM)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

***/

#include "saoriviewentry.h"
#include "saoriapplication.h"
#include "saoricache.h"
#include <QRegExp>
#include <QDebug>

SaoriViewEntry::SaoriViewEntry(qlonglong id, QWidget *parent) :
    QTextBrowser(parent)
{
    setReadOnly(true);
    setWordWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
    setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
    setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
    setOpenLinks(false);
    setOpenExternalLinks(false);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_id = id;
    connect(SaoriApplication::saori()->cache(),&SaoriCache::downloaded,this,&SaoriViewEntry::downloaded);
}

void SaoriViewEntry::setContent(const QString content)
{
    m_original = content;
    QStringList imgs;
    for(int i = 0;(i = m_original.indexOf("<img(",i)) != -1;++ i) {
        int p = m_original.indexOf(')',i);
        imgs << m_original.mid(i + 5,p - (i + 5));
    }
    for (auto i:imgs) {
        m_urlmap[i] = SaoriApplication::saori()->cache()->fileCache(QUrl(i));
    }
    setText(designedText());
    update();
    document()->adjustSize();
    setFixedHeight(document()->size().height());
}

qlonglong SaoriViewEntry::id()
{
    return m_id;
}

void SaoriViewEntry::resizeEvent(QResizeEvent *e)
{
    document()->adjustSize();
    setFixedHeight(document()->size().height() + 10);
    QTextBrowser::resizeEvent(e);
}

const QString SaoriViewEntry::designedText()
{
    QString result;
    result += "<head><link rel=\"stylesheet\" type=\"text/css\" href=\":/css/saoristyle.css\" /></head>";
    result += "<body>";
    result += imageReplacer();
    result += "</body>";
    return result;
}

const QString SaoriViewEntry::imageReplacer()
{
    QString result = m_original;
    for (auto k:m_urlmap.keys()) {
        result.replace(QString("<img(" + k + ")"),
                       QString("<img src=\"" + m_urlmap[k] + "\""));
    }
    return result;
}

void SaoriViewEntry::downloaded(const QUrl url)
{
    m_urlmap[url.toString()] = SaoriApplication::saori()->cache()->fileCache(url);
    setText(designedText());
    for (auto i:m_urlmap) {
        if (i.indexOf(":/")) return;
    }
    disconnect(SaoriApplication::saori()->cache(),&SaoriCache::downloaded,this,&SaoriViewEntry::downloaded);
}
